@@ -8,7 +8,7 @@
 #
 # Returns:
 #   A vector of length nrow(samples) containing the pointwise log-likelihood for the nth observation 
-loglik_gaussian <- function(n, data, samples, link) {
+loglik_gaussian <- function(n, data, samples, link = "identity") {
   sigma <- get_sigma(samples$sigma, data = data, method = "logLik", n = n)
   args <- list(mean = ilink(samples$eta[, n], link), sd = sigma)
   # censor_loglik computes the conventional loglik in case of no censoring 
@@ -17,7 +17,7 @@ loglik_gaussian <- function(n, data, samples, link) {
   weight_loglik(out, n = n, data = data)
 }
 
-loglik_student <- function(n, data, samples, link) {
+loglik_student <- function(n, data, samples, link = "identity") {
   sigma <- get_sigma(samples$sigma, data = data, method = "logLik", n = n)
   args <- list(df = samples$nu, mu = ilink(samples$eta[, n], link), 
                sigma = sigma)
@@ -26,7 +26,7 @@ loglik_student <- function(n, data, samples, link) {
   weight_loglik(out, n = n, data = data)
 }
 
-loglik_cauchy <- function(n, data, samples, link) {
+loglik_cauchy <- function(n, data, samples, link = "identity") {
   sigma <- get_sigma(samples$sigma, data = data, method = "logLik", n = n)
   args <- list(df = 1, mu = ilink(samples$eta[, n], link), 
                sigma = sigma)
@@ -35,7 +35,7 @@ loglik_cauchy <- function(n, data, samples, link) {
   weight_loglik(out, n = n, data = data)
 }
 
-loglik_lognormal <- function(n, data, samples, link) {
+loglik_lognormal <- function(n, data, samples, link = "identity") {
   # link is currently ignored for lognormal models
   # as 'identity' is the only valid link
   sigma <- get_sigma(samples$sigma, data = data, method = "logLik", n = n)
@@ -45,42 +45,42 @@ loglik_lognormal <- function(n, data, samples, link) {
   weight_loglik(out, n = n, data = data)
 }
 
-loglik_multi_gaussian <- function(n, data, samples, link) {
+loglik_multi_gaussian <- function(n, data, samples, link = "identity") {
   nobs <- data$N_trait * data$K_trait
   nsamples <- nrow(samples$eta)
   obs <- seq(n, nobs, data$N_trait)
   out <- sapply(1:nsamples, function(i) 
     dmulti_normal(data$Y[n,], Sigma = samples$Sigma[i, , ], log = TRUE,
-                 mu = ilink(samples$eta[i, obs], link)))
+                  mu = ilink(samples$eta[i, obs], link)))
   # no truncation allowed
   weight_loglik(out, n = n, data = data)
 }
 
-loglik_multi_student <- function(n, data, samples, link) {
+loglik_multi_student <- function(n, data, samples, link = "identity") {
   nobs <- data$N_trait * data$K_trait
   nsamples <- nrow(samples$eta)
   obs <- seq(n, nobs, data$N_trait)
   out <- sapply(1:nsamples, function(i) 
     dmulti_student(data$Y[n,], df = samples$nu[i, ], 
-                  Sigma = samples$Sigma[i, , ], log = TRUE,
-                  mu = ilink(samples$eta[i, obs], link)))
+                   Sigma = samples$Sigma[i, , ], log = TRUE,
+                   mu = ilink(samples$eta[i, obs], link)))
   # no truncation allowed
   weight_loglik(out, n = n, data = data)
 }
 
-loglik_multi_cauchy <- function(n, data, samples, link) {
+loglik_multi_cauchy <- function(n, data, samples, link = "identity") {
   nobs <- data$N_trait * data$K_trait
   nsamples <- nrow(samples$eta)
   obs <- seq(n, nobs, data$N_trait)
   out <- sapply(1:nsamples, function(i) 
     dmulti_student(data$Y[n,], df = 1, 
-                  Sigma = samples$Sigma[i, , ], log = TRUE,
-                  mu = ilink(samples$eta[i, obs], link)))
+                   Sigma = samples$Sigma[i, , ], log = TRUE,
+                   mu = ilink(samples$eta[i, obs], link)))
   # no truncation allowed
   weight_loglik(out, n = n, data = data)
 }
 
-loglik_gaussian_cov <- function(n, data, samples, link) {
+loglik_gaussian_cov <- function(n, data, samples, link = "identity") {
   # currently, only ARMA1 processes are implemented
   rows <- with(data, begin_tg[n]:(begin_tg[n] + nrows_tg[n] - 1))
   Y_part <- data$Y[rows]
@@ -108,7 +108,7 @@ loglik_gaussian_cov <- function(n, data, samples, link) {
   out
 }
 
-loglik_student_cov <- function(n, data, samples, link) {
+loglik_student_cov <- function(n, data, samples, link = "identity") {
   # currently, only ARMA1 processes are implemented
   rows <- with(data, begin_tg[n]:(begin_tg[n] + nrows_tg[n] - 1))
   Y_part <- data$Y[rows]
@@ -137,13 +137,12 @@ loglik_student_cov <- function(n, data, samples, link) {
   out
 }
 
-loglik_cauchy_cov <- function(n, data, samples, link) {
+loglik_cauchy_cov <- function(n, data, samples, link = "identity") {
   samples$nu <- matrix(rep(1, nrow(samples$eta)))
   loglik_student_cov(n = n, data = data, samples = samples, link = link)
 }
                   
-
-loglik_binomial <- function(n, data, samples, link) {
+loglik_binomial <- function(n, data, samples, link = "logit") {
   trials <- ifelse(length(data$max_obs) > 1, data$max_obs[n], data$max_obs) 
   args <- list(size = trials, prob = ilink(samples$eta[, n], link))
   out <- censor_loglik(dist = "binom", args = args, n = n, data = data)
@@ -151,42 +150,55 @@ loglik_binomial <- function(n, data, samples, link) {
   weight_loglik(out, n = n, data = data)
 }  
 
-loglik_bernoulli <- function(n, data, samples, link) {
-  args <- list(size = 1, prob = ilink(samples$eta[, n], link))
+loglik_bernoulli <- function(n, data, samples, link = "logit") {
+  if (!is.null(data$N_trait)) {  # 2PL model
+    eta <- samples$eta[, n] * exp(samples$eta[, n + data$N_trait])
+  } else {
+    eta <- samples$eta[, n]
+  }
+  args <- list(size = 1, prob = ilink(eta, link))
   out <- censor_loglik(dist = "binom", args = args, n = n, data = data)
   # no truncation allowed
-  weight_loglik(out, n = n, data = data)
+  out <- weight_loglik(out, n = n, data = data)
+  is_nan <- is.nan(out)
+  if (any(is_nan)) {
+    # for 2PL models NaN may occure for numerical reasons
+    warning(paste("observation", n, "had", length(which(is_nan)), 
+                  "logLik samples that were NaN"))
+    out[is_nan] <- mean(out[!is_nan])
+  }
+  out
 }
 
-loglik_poisson <- function(n, data, samples, link) {
+loglik_poisson <- function(n, data, samples, link = "log") {
   args <- list(lambda = ilink(samples$eta[, n], link))
   out <- censor_loglik(dist = "pois", args = args, n = n, data = data)
   out <- truncate_loglik(out, cdf = ppois, args = args, data = data)
   weight_loglik(out, n = n, data = data)
 }
 
-loglik_negbinomial <- function(n, data, samples, link) {
+loglik_negbinomial <- function(n, data, samples, link = "log") {
   args <- list(mu = ilink(samples$eta[, n], link), size = samples$shape)
   out <- censor_loglik(dist = "nbinom", args = args, n = n, data = data)
   out <- truncate_loglik(out, cdf = pnbinom, args = args, data = data)
   weight_loglik(out, n = n, data = data)
 }
 
-loglik_geometric <- function(n, data, samples, link) {
+loglik_geometric <- function(n, data, samples, link = "log") {
   args <- list(mu = ilink(samples$eta[, n], link), size = 1)
   out <- censor_loglik(dist = "nbinom", args = args, n = n, data = data)
   out <- truncate_loglik(out, cdf = pnbinom, args = args, data = data)
   weight_loglik(out, n = n, data = data)
 }
 
-loglik_exponential <-  function(n, data, samples, link) {
+loglik_exponential <-  function(n, data, samples, link = "log") {
   args <- list(rate = 1 / ilink(samples$eta[, n], link))
   out <- censor_loglik(dist = "exp", args = args, n = n, data = data)
   out <- truncate_loglik(out, cdf = pexp, args = args, data = data)
   weight_loglik(out, n = n, data = data)
 }
 
-loglik_gamma <- function(n, data, samples, link) {
+loglik_gamma <- function(n, data, samples, link = "inverse") {
   args <- list(shape = samples$shape,
                scale = ilink(samples$eta[, n], link) / samples$shape)
   out <- censor_loglik(dist = "gamma", args = args, n = n, data = data)
@@ -194,7 +206,7 @@ loglik_gamma <- function(n, data, samples, link) {
   weight_loglik(out, n = n, data = data)
 }
 
-loglik_weibull <- function(n, data, samples, link) {
+loglik_weibull <- function(n, data, samples, link = "log") {
   args <- list(shape = samples$shape,
                scale = ilink(samples$eta[, n] / samples$shape, link))
   out <- censor_loglik(dist = "weibull", args = args, n = n, data = data)
@@ -202,14 +214,22 @@ loglik_weibull <- function(n, data, samples, link) {
   weight_loglik(out, n = n, data = data)
 }
 
-loglik_inverse.gaussian <- function(n, data, samples, link) {
+loglik_inverse.gaussian <- function(n, data, samples, link = "1/mu^2") {
   args <- list(mean = ilink(samples$eta[, n], link), shape = samples$shape)
   out <- censor_loglik(dist = "invgauss", args = args, n = n, data = data)
   out <- truncate_loglik(out, cdf = pinvgauss, args = args, data = data)
   weight_loglik(out, n = n, data = data)
 }
 
-loglik_hurdle_poisson <- function(n, data, samples, link) {
+loglik_beta <- function(n, data, samples, link = "logit") {
+  mu <- ilink(samples$eta[, n], link)
+  args <- list(shape1 = mu * samples$phi, shape2 = (1 - mu) * samples$phi)
+  out <- censor_loglik(dist = "beta", args = args, n = n, data = data)
+  out <- truncate_loglik(out, cdf = pbeta, args = args, data = data)
+  weight_loglik(out, n = n, data = data)
+}
+
+loglik_hurdle_poisson <- function(n, data, samples, link = "log") {
   theta <- ilink(samples$eta[, n + data$N_trait], "logit")
   args <- list(lambda = ilink(samples$eta[, n], link))
   out <- hurdle_loglik(pdf = dpois, theta = theta, 
@@ -217,7 +237,7 @@ loglik_hurdle_poisson <- function(n, data, samples, link) {
   weight_loglik(out, n = n, data = data)
 }
 
-loglik_hurdle_negbinomial <- function(n, data, samples, link) {
+loglik_hurdle_negbinomial <- function(n, data, samples, link = "log") {
   theta <- ilink(samples$eta[, n + data$N_trait], "logit")
   args <- list(mu = ilink(samples$eta[, n], link), size = samples$shape)
   out <- hurdle_loglik(pdf = dnbinom, theta = theta, 
@@ -225,7 +245,7 @@ loglik_hurdle_negbinomial <- function(n, data, samples, link) {
   weight_loglik(out, n = n, data = data)
 }
 
-loglik_hurdle_gamma <- function(n, data, samples, link) {
+loglik_hurdle_gamma <- function(n, data, samples, link = "log") {
   theta <- ilink(samples$eta[, n + data$N_trait], "logit")
   args <- list(shape = samples$shape, 
                scale = ilink(samples$eta[, n], link) / samples$shape)
@@ -234,7 +254,7 @@ loglik_hurdle_gamma <- function(n, data, samples, link) {
   weight_loglik(out, n = n, data = data)
 }
 
-loglik_zero_inflated_poisson <- function(n, data, samples, link) {
+loglik_zero_inflated_poisson <- function(n, data, samples, link = "log") {
   theta <- ilink(samples$eta[, n + data$N_trait], "logit")
   args <- list(lambda = ilink(samples$eta[, n], link))
   out <- zero_inflated_loglik(pdf = dpois, theta = theta, 
@@ -242,7 +262,7 @@ loglik_zero_inflated_poisson <- function(n, data, samples, link) {
   weight_loglik(out, n = n, data = data)
 }
 
-loglik_zero_inflated_negbinomial <- function(n, data, samples, link) {
+loglik_zero_inflated_negbinomial <- function(n, data, samples, link = "log") {
   theta <- ilink(samples$eta[, n + data$N_trait], "logit")
   args <- list(mu = ilink(samples$eta[, n], link), size = samples$shape)
   out <- zero_inflated_loglik(pdf = dnbinom, theta = theta, 
@@ -250,7 +270,16 @@ loglik_zero_inflated_negbinomial <- function(n, data, samples, link) {
   weight_loglik(out, n = n, data = data)
 }
 
-loglik_categorical <- function(n, data, samples, link) {
+loglik_zero_inflated_binomial <- function(n, data, samples, link = "log") {
+  trials <- ifelse(length(data$max_obs) > 1, data$max_obs[n], data$max_obs) 
+  theta <- ilink(samples$eta[, n + data$N_trait], "logit")
+  args <- list(size = trials, prob = ilink(samples$eta[, n], link))
+  out <- zero_inflated_loglik(pdf = dbinom, theta = theta, 
+                              args = args, n = n, data = data)
+  weight_loglik(out, n = n, data = data)
+}
+
+loglik_categorical <- function(n, data, samples, link = "logit") {
   ncat <- ifelse(length(data$max_obs) > 1, data$max_obs[n], data$max_obs) 
   if (link == "logit") {
     p <- cbind(rep(0, nrow(samples$eta)), samples$eta[, n, 1:(ncat - 1)])
@@ -259,7 +288,7 @@ loglik_categorical <- function(n, data, samples, link) {
   weight_loglik(out, n = n, data = data)
 }
 
-loglik_cumulative <- function(n, data, samples, link) {
+loglik_cumulative <- function(n, data, samples, link = "logit") {
   ncat <- ifelse(length(data$max_obs) > 1, data$max_obs[n], data$max_obs) 
   y <- data$Y[n]
   if (y == 1) { 
@@ -273,7 +302,7 @@ loglik_cumulative <- function(n, data, samples, link) {
   weight_loglik(out, n = n, data = data)
 }
 
-loglik_sratio <- function(n, data, samples, link) {
+loglik_sratio <- function(n, data, samples, link = "logit") {
   ncat <- ifelse(length(data$max_obs) > 1, data$max_obs[n], data$max_obs) 
   y <- data$Y[n]
   q <- sapply(1:min(y, ncat - 1), function(k) 
@@ -290,7 +319,7 @@ loglik_sratio <- function(n, data, samples, link) {
   weight_loglik(out, n = n, data = data)
 }
 
-loglik_cratio <- function(n, data, samples, link) {
+loglik_cratio <- function(n, data, samples, link = "logit") {
   ncat <- ifelse(length(data$max_obs) > 1, data$max_obs[n], data$max_obs) 
   y <- data$Y[n]
   q <- sapply(1:min(y, ncat-1), function(k) 
@@ -307,7 +336,7 @@ loglik_cratio <- function(n, data, samples, link) {
   weight_loglik(out, n = n, data = data)
 }
 
-loglik_acat <- function(n, data, samples, link) {
+loglik_acat <- function(n, data, samples, link = "logit") {
   ncat <- ifelse(length(data$max_obs) > 1, data$max_obs[n], data$max_obs) 
   y <- data$Y[n]
   if (link == "logit") { # more efficient calculation 
